################################################################################
# author: Jona Cederbaum (with thanks to face author Luo Xiao)
# NOTE: this code is partly copied from the help page of face.sparse.
################################################################################
# description: CD4 data analysis using FACE or FACE-STEP-1
################################################################################

###############
# load packages
###############
require(refund)
require(face)

###########################
# create folder for results
###########################
dir.create("results")

###########################
# generate name for results
# depending on the date
###########################
dat_start <- format(Sys.time(), "_%d_%b")

################################################################################
# specify input parameters 
################################################################################

###################
# general smoothing
###################
d_grid <- 100           # pre-specified grid length for equidistant grid on which the mean, the covariance surfaces, the eigenfunctions
                        # and the functional random effects are evaluated
                        # NOTE: the length of the grid can be important for computation time
min_grid <- 0           # minimum value of eqidistant grid (should approx. correspond to minimum value of time interval)
max_grid <- 1           # maximum value of eqidistant grid (should approx. correspond to maximum value of time interval)

################################
# mean and covariance estimation
################################
bf <- 13                     # dimension of basis (number of basis functions) used for mean and covariance estimation
m <- list(c(2, 2))           # marginal spline order and marginal order of the penalty for mean and covariance estimation
                             # should be a list of length one
use_step_1_only <- FALSE     # TRUE to estimate using FACE-STEP-1, i.e. only the first iteration is used

################
# FPC estimation
################
var_level <- 0.99            # pre-specified level of variance used for the choice of the number of the FPCs

#########################
# load data and
# bring into right format
#########################
data(cd4)
n <- nrow(cd4)
D <- ncol(cd4)

range <- range(as.numeric(colnames(cd4)))

id <- rep(1:n,each=D)
t <- rep(min(range):max(range),times=n)

#######################################
# transform time to [0, 1]
# such that face.sparse() is applicable
#######################################
trans_fun <- function(x, min, max){y <- (x - min) / (max - min); y}

t_trans <- trans_fun(x = t, min = min(range), max = max(range))

y <- as.vector(sqrt(t(cd4)))
isNA <- which(is.na(y))

dat_cd4 <- data.frame(y = y[-isNA], argvals = t_trans[-isNA], subj = id[-isNA])


#######################################
# adapt input parameters to face.sparse
#######################################

# get my_grid (corresponding to argvals.new) from min_grid, max_grid and d_grid
my_grid <- seq(min_grid, max_grid, length.out = d_grid)

# determine number of knots from number of 
# B-splines and use degree instead of order
degree_covs <- m[[1]][[1]] + 1   # get degree instead of order
k <- bf - degree_covs   # number of knots
   

###################
# initialize output
###################
output <- list()

################################################################################
# start estimation
################################################################################

if(!use_step_1_only){
  output[["time_all"]] <- 
    system.time(results_face <- 
                  face.sparse(dat_cd4, argvals.new = my_grid, center = TRUE, knots = k, knots.option = "equally-spaced",
                              p = degree_covs, m = m[[1]][2], calculate.scores = TRUE, pve = var_level, newdata = dat_cd4))
}else{
  output[["time_all"]] <- 
    system.time(results_face <- 
                  face:::face.sparse.inner(dat_cd4, argvals.new = my_grid, center = TRUE, knots = k, knots.option = "equally-spaced",
                              p = degree_covs, m = m[[1]][2], calculate.scores = TRUE, pve = var_level, newdata = dat_cd4))
}


################################
# re-scale eigenfunctions and 
# eigenvalues to grid (-18,42)
################################
trans_ratio <- max(range) - min(range)
results_face$eigenvalues <- results_face$eigenvalues * trans_ratio
for(k in 1:length(results_face$eigenvalues)){
  results_face$eigenfunctions[, k] <- results_face$eigenfunctions[, k] * (1 / sqrt(trans_ratio))
}
output[["results_face"]]<-results_face


#################
# get sessionInfo
#################
sessionInfo <- sessionInfo()

############
# save input
############
if(!use_step_1_only){
  save(d_grid, bf, m, var_level, dat_cd4, min_grid, max_grid, use_step_1_only, 
       file = paste0("results/input_face", dat_start, ".Rdata"))
}else{
  save(d_grid, bf, m, var_level, dat_cd4, min_grid, max_grid, use_step_1_only, 
       file = paste0("results/input_face_step_1", dat_start, ".Rdata"))
  
}

################################################################################
# output parameters 
################################################################################
# output is a list of two elements:

## time_all: the total system.time() for calling function face.sparse()

## results: a list of all estimates further described in ?face.sparse


#############
# save output
#############
if(!use_step_1_only){
  saveRDS(output, file = paste0("results/results_face", dat_start, ".rds"))  
}else{
  saveRDS(output, file = paste0("results/results_face_step_1", dat_start, ".rds"))  
}

################################################################################
