##############################################################################################################
# author: Jona Cederbaum
##############################################################################################################
# description: analyze application results for CD4 data using our smoothing approach or WHOLE, TRI
# NOTE: although the fRI is called B in the estimation, we rename it to E to make it consistent with the paper
##############################################################################################################

###############
# load packages
###############
require(data.table)
require(scales)

##################
# source functions
#################
source("../useful_functions.R")

########################
# define plot properties
########################
cex_use <- 2            # size in plots
cex.axis_use <- 2       # size of axes
cex.lab_use <- 3        # size of labels
cex.main_use <- 3       # size of main
main_use <- ""          # plot title
line_use <- 4           # line in which the labels of the axes should be
lwd_use <- 3            # line width
cex_legend_use <- 1.8   # legend text size
cex_legend_use<-1.8     # legend text size

###############
# which results
###############
dat_start <- "_26_Aug"  # date on which the data analysis was started.

###################
# which covariances
# estimations
###################
which_covs <- c("whole", "tri", "tri_constr", "tri_constr_weights")

##############
# load results
##############
res <- readRDS(paste("results/results", dat_start, ".rds", sep = ""))

##############################
# extract estimated components
##############################
results <- res$results
res_cov <- list()
res_fpc <- list()
for(i in seq_along(which_covs)){
  res_cov[[i]] <- results[[paste0("cov_hat_", which_covs[i])]]
  res_fpc[[i]] <- results[[paste0("fpc_hat_", which_covs[i])]]
}


##################################
# extract data and evaluation grid
##################################
curve_info <- res$curve_info
my_grid <- results[["my_grid"]]

################################################################################
# make plots for all smoothing approaches separately
################################################################################

for(i in seq_along(which_covs)){
  ##############################
  # extract estimated components
  ##############################
  res_fpc_use <- res_fpc[[i]]
  res_cov_use <- res_cov[[i]]
  N_E_hat <- res_fpc_use$N_B
  nu_E_hat <- res_fpc_use$nu_B_hat
  sigmasq_hat <- res_cov_use$sigmasq
  phi_E_hat_grid <- res_fpc_use$phi_B_hat_grid
  cov_E_hat <- get_covs_fun(N = N_E_hat, phi = phi_E_hat_grid, nu = nu_E_hat)
  
  ########################
  # plot eigenfunctions
  ########################
  pdf(paste0("results/FPCs_estimated_", which_covs[i], dat_start, ".pdf"))
  par(mar = c(5.5, 7.8, 4, 2))
  col_FPCs_E <- c(rep("black", 2), rep(alpha("black", 0.5), 9))
  lwd_use_E <- c(rep(lwd_use, 2), rep((lwd_use - 1), 9))
  matplot(my_grid, phi_E_hat_grid, t="l", lwd = lwd_use_E, main="", 
          ylab="", xlab="", cex.axis = cex.axis_use, cex.lab = cex.lab_use, 
          cex.main = cex.main_use, col = col_FPCs_E, ylim = c(-0.4, 0.4))
  mtext(side = 1, "time (since SC)", line = line_use, cex = cex.lab_use)
  mtext(side = 2, expression(paste(phi["k"]^E, "(t)")), line = line_use, cex = cex.lab_use)
  
  if(N_E_hat >= 2){
    N_E_hat_use <- 2
  }else{
    N_E_hat_use <- N_E_hat
  }
  legend <- NULL
  for(k in 1:N_E_hat_use){
    legend <- cbind(legend, paste0("k=", k))
  }
  legend("topright", lty = seq_len(N_E_hat), legend = legend, 
         bty = "n", cex = cex_legend_use, lwd = lwd_use)
  abline(h = 0, lty = 3)
  dev.off()
  
  #########################
  # plot covariance surface
  #########################
  pdf(paste0("results/cov_estimated_persp_image_", which_covs[i], dat_start, ".pdf"))
  par(mar = c(5.5, 6.2, 4, 2))
  
  ############
  # persp plot
  
  # make z thinner for better display
  thinner <- seq(1, nrow(cov_E_hat), by = 4) # do not use all grid points to make plot with 'border' nicer
  z_plotting <- cov_E_hat[thinner, thinner]
  
  persp(z_plotting, ticktype = "detailed", xlab = "", ylab = "", zlab = "", theta = 30, phi = 38,
        xlim = c(0, 1), ylim = c(0, 1), zlim = c(0, 40), col = get_color_persp_fun(z_plotting),
        cex.axis = cex.axis_use)
  
  ############
  # image plot
  
  image(my_grid, my_grid, cov_E_hat, xlab = "", ylab = "", 
        col = terrain.colors(ncol(cov_E_hat)), cex.axis = cex.axis_use, cex.lab = cex.lab_use)
  contour(my_grid, my_grid, cov_E_hat, add = T)
  mtext(side = 1, "time (since SC)", line = line_use, cex = cex.lab_use)
  mtext(side = 2, "time (since SC)", line = line_use, cex = cex.lab_use)
  dev.off()
  
  
}
################################################################################
