################################################################################################################
# author: Jona Cederbaum
################################################################################################################
# description: function that calls all subfunctions used for the estimation of the FLMM and returns the result.
################################################################################################################
call_functions_fun <- function(curve_info, use_RI = FALSE, use_simple = FALSE, method = "REML", 
                               use_bam = TRUE, bs = "ps", d_grid = 100, mp = TRUE, 
                               para_estim = FALSE, para_estim_nc, min_grid = 0, 
                               max_grid = 1, my_grid = NULL, bf_mean = 10, 
                               bf_covariates = 10,
                               m_mean = c(2,3), covariate = FALSE, covariate_form, num_covariates, 
                               interaction, which_interaction,
                               save_model_mean = FALSE,
                               bf_covs, m_covs,
                               use_whole = FALSE, use_tri = FALSE, use_tri_constr = TRUE,
                               use_tri_constr_weights = FALSE, np = TRUE, var_level = 0.95, 
                               N_B = NA, N_C = NA, N_E = NA, 
                               use_famm = FALSE, use_bam_famm = TRUE,
                               bs_int_famm = list(bs = "ps", k = 8, m = c(2, 3)), 
                               bs_y_famm = list(bs = "ps", k = 8, m = c(2, 3)),
                               para_estim_famm = FALSE, para_estim_famm_nc,
                               save_model_famm = FALSE){
  
  ##############################################################################
  # preparations
  ##############################################################################
  
  #####################
  # checks and warnings
  #####################
  if(packageVersion(pkg="data.table") == "1.9.4")
    stop("you are using data.table version 1.9.4 which cannot be used due to internal bugs,
         please install a different version")
  
  if(packageVersion(pkg="mgcv") == "1.8-12")
    warning("you are using mgcv version 1.8-12 for which errors might occur. 
            In case of problems, please use a different version")
  
  if(!is.data.table(curve_info))
    stop("curve_info needs to be a data.table, see ?data.table")
  
  if(bs != "ps")
    stop("so far only P-splines are allowed")
  
  if(is.na(var_level) & (is.na(N_B) | is.na(N_E)))
    stop("either var_level or N_B, N_C, and N_E have to be specified")
  
  if(var_level > 1 & (is.na(N_B) | is.na(N_C) | is.na(N_E)))
    stop("var_level has to be smaller than 1")
  
  if(covariate & (length(covariate_form) != num_covariates))
    stop("covariate_form has to be of length ", num_covariates)
  
  if(interaction & ((nrow(which_interaction) != num_covariates) | (ncol(which_interaction) != num_covariates)))
    stop("which_interaction has to be a matrix of dimension ", num_covariates, "x", num_covariates)
  
  if(!isSymmetric(which_interaction))
    stop("which_interaction needs to be symmetric")
  
  if(covariate & (length(bf_mean) != 1))
    stop(" bf_mean needs to be a number vector of length 1")
  
  if(covariate & (length(bf_covariates) != 1))
    stop("so far, the same number of basis functions is used for all covariate and interaction effects, 
         please set bf_covariates to a numeric vector of length 1")
  
  if(!use_RI & (length(bf_covs) != 3))
    stop("for use_RI == FALSE, bf_covs has to be of length 3")
  
  if(use_RI & (!use_simple) & ((length(bf_covs) != 2)))
    stop("for use_RI == TRUE and use_simple == FALSE, bf_covs has to be of length 2")
  
  if(use_simple & ((length(bf_covs) != 1)))
    stop("for use_simple == TRUE, bf_covs has to be of length 1")
  
  if(!is.list(m_covs))
    stop("m_covs needs to be a list")
  
  if(!use_RI & ((length(m_covs) != 3)))
    stop(" for use_RI == FALSE, m_covs needs to be list of length 3")
  
  if(use_RI & (!use_simple) & ((length(m_covs) != 2)))
    stop(" for use_RI == TRUE and use_simple == FALSE, m_covs needs to be list of length 2")
  
  if(use_simple & (length(m_covs) != 1))
    stop(" for use_simple == TRUE, m_covs needs to be list of length 1")
  
  if(!use_RI){
    if(((length(m_covs[[1]]) != 2)) | ((length(m_covs[[2]]) != 2)) | ((length(m_covs[[3]]) != 2)))
      stop("all three entries of the list m_covs need to be vectors of 2 numerical entries")
  }
  
  if(use_RI & (!use_simple)){
    if(((length(m_covs[[1]]) != 2)) | ((length(m_covs[[2]]) != 2)))
    stop("all two entries of the list m_covs need to be vectors of 2 numerical entries")
  } 
  
  if(use_simple & (length(m_covs[[1]]) != 2))
    stop("m_covs need to be lists of a vector of 2 numerical entries")
  
  if(!use_bam)
    warning("note that when use_bam == FALSE, computation time can be large")
  
  if(max_grid <= min_grid)
    stop("min_grid needs to be smaller than max_grid")
  
  if(!use_famm)
    warning("as use_famm == FALSE, no confidence bands for mean and covariate effects are given out")
  
  if(use_famm & !is.list(bs_int_famm))
    stop("bs_int_famm has to be a list")
  
  if(use_famm & !is.list(bs_y_famm))
    stop("bs_y_famm has to be a list")
  
  if(use_famm & (length(bs_int_famm) != 3))
    stop("bs_int_famm has to be a list of length 3")
  
  if(use_famm & (length(bs_y_famm) != 3))
    stop("bs_int_famm has to be a list of length 3")
  
  if(use_famm & (length(bs_int_famm$k) != 1))
    stop("bs_int_famm$k has to be of length 1")
  
  if(use_famm & (length(bs_y_famm$k) != 1))
    stop("bs_y_famm$k has to be of length 1")
  
  if(use_famm & (bs_int_famm$bs == "ps") & (length(bs_int_famm$m) != 2))
    stop("when bs_int_famm$bs == 'ps' is used, bs_int_famm$m has to be of length 2")
  
  if(use_famm & (bs_y_famm$bs == "ps") & (length(bs_y_famm$m) != 2))
    stop("when bs_y_famm$bs == 'ps' is used, bs_y_famm$m has to be of length 2")
  
  if(!use_RI & use_simple)
    stop("when using use_simple == TRUE, please also set use_RI to TRUE")
  
  if(!is.integer(curve_info$n_long))
    stop("in curve_info, n_long needs to be an integer")
  
  if(!is.integer(curve_info$subject_long))
    stop("in curve_info, subject_long needs to be an integer")
  
  if(!use_RI & (!is.integer(curve_info$word_long)))
    stop("in curve_info, word_long needs to be an integer")
  
  
  ###################
  # initialize output
  ###################
  res <- list()   
  
  ##################################
  # get levels of grouping variables
  ##################################
  I <- length(unique(curve_info$subject_long)) # number of levels of first grouping variable (e.g. speaker)
  if(!use_RI){
    J <- length(unique(curve_info$word_long))  # number of levels of second grouping variable (e.g. target words)
  }else{
    J <- NA # in case of one fRI, J is automatically set to NA
  }
  
  # renumberlevels of the grouping
  # variables
  ######################################
  
  # renumber curves
  n <- length(unique(curve_info$n_long))
  n_orig <- unique(curve_info$n_long)
  curve_info[, n_long_orig := n_long]
  for(i in seq_along(n_orig)){
    curve_info[n_long_orig == n_orig[i], n_long := (1:n)[i]]
  }
  res[["n_orig"]] <- n_orig
  
  # renumber levels of first grouping variable (e.g. speakers)
  subject_orig <- unique(curve_info$subject_long)
  curve_info[, subject_long_orig := subject_long]
  for(i in seq_along(subject_orig)){
    curve_info[subject_long_orig == subject_orig[i], subject_long := (1:I)[i]]
  }
  res[["subject_orig"]] <- subject_orig
  
  # renumber levels of second grouping variable (e.g. target words)
  if(!use_RI){
    word_orig <- unique(curve_info$word_long)  
    curve_info[, word_long_orig := word_long]
    for(i in seq_along(word_orig)){
      curve_info[word_long_orig == word_orig[i], word_long := (1:J)[i]]
    }
    res[["word_orig"]] <- word_orig
  }else{
    res[["word_orig"]] <- NULL
  }
  
  ##########################################
  # extract measurement points of curve_info
  ##########################################
  t <- curve_info$t 
  
  ##############
  # specify grid
  ##############  
  if(is.null(my_grid)){
    # compute grid on which the covariance will later be evaluated
    res[["my_grid"]] <- seq(from = min_grid, to = max_grid, length = d_grid)  
  }else{
    res[["my_grid"]] <- my_grid
    d_grid <- length(my_grid)
    if(!isTRUE(all.equal(0, var(diff(res[["my_grid"]]))))){
      warning("the evaluation grid is not equidistant")
    }
  }
  
  ##############################################################################
  ##############################################################################
  # estimations
  ##############################################################################
  ##############################################################################
  
  ##############################################################################
  # estimate mean function
  ##############################################################################
  cat("mean estimation", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
  res[["mean_hat"]] <- estimate_mean_fun(bf = bf_mean, bf_covariates = bf_covariates, method = method, 
                                         covariate = covariate,num_covariates = num_covariates, 
                                         covariate_form = covariate_form, 
                                         save_model_mean = save_model_mean, n = n, my_grid = res[["my_grid"]], bs = bs, 
                                         m = m_mean, curve_info = curve_info, interaction = interaction, 
                                         which_interaction = which_interaction, use_bam = use_bam) 
  
  ########################################
  # preparations for covariance estimation
  ########################################
  if(any(!is.na(res[["mean_hat"]][["y_tilde"]]))){
    y_tilde <- res[["mean_hat"]][["y_tilde"]]
    
    #########################
    # take out y_tilde 
    # from res[["mean_hat]]
    # and add in curve_info
    #########################
    res[["mean_hat"]][["y_tilde"]] <- NULL
    curve_info[, y_tilde := y_tilde]
    
    ####################
    # get cross products
    ####################
    cat("get cross products", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
    curve_info[, id := 1:nrow(curve_info)]
    preps <- get_crossprods_fun(y_tilde = y_tilde, curve_info = curve_info, 
                                my_grid = res[["my_grid"]], d_grid = d_grid, use_RI = use_RI, I = I, J = J, 
                                t = t)
    set(curve_info, i = NULL, "id", NULL) 
    index <- preps$index
    set(index, i = NULL, "id2", NULL)
    
    grid_row <- preps$grid_row
    grid_col <- preps$grid_col
    
    same_subject_grid <- preps$same_subject_grid
    if(!use_RI)
      same_word_grid <- preps$same_word_grid
    same_curve_grid <- preps$same_curve_grid
    same_point_grid <- preps$same_point_grid
    
    preps <- NULL
    gc()
    
    ############################################################################
    # estimation of the covariances
    ############################################################################
    if(use_RI)
      same_word_grid <- NA 
    
    #################################
    # whole: using all cross products
    # without symmetry constraint
    #################################
    if(use_whole){
      cat("covariance estimation whole", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
      res[["time_cov_whole"]] <- 
        system.time(res[["cov_hat_whole"]] <- 
                      estimate_cov_whole_fun(index = index, bf = bf_covs, method = method, 
                                             grid_col = grid_col, grid_row = grid_row, d_grid = d_grid, 
                                             bs = bs, m = m_covs, use_bam = use_bam, t = t, 
                                             same_subject_grid = same_subject_grid, 
                                             same_word_grid = same_word_grid, same_curve_grid = same_curve_grid, 
                                             same_point_grid = same_point_grid, 
                                             mp = mp, para_estim = para_estim, para_estim_nc = para_estim_nc, 
                                             use_RI = use_RI, weights = NULL, use_simple = use_simple, np = np)) 
    }
    
    
    #######################
    # select cross products
    # on upper triangle
    #######################
    if(use_tri | use_tri_constr | use_tri_constr_weights)
      index_upperTri <- index[row_t_bivariate <= col_t_bivariate, ]
    
    #############################
    # tri: using triangle only 
    # without symmetry constraint
    #############################
    if(use_tri){
      cat("covariance estimation tri", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
      res[["time_cov_tri"]] <- 
        system.time(res[["cov_hat_tri"]] <- 
                      estimate_cov_tri_fun(index_upperTri = index_upperTri, bf = bf_covs, method = method, 
                                           grid_col = grid_col, grid_row = grid_row, d_grid = d_grid, 
                                           bs = bs, m = m_covs, use_bam = use_bam, t = t, 
                                           same_subject_grid = same_subject_grid, 
                                           same_word_grid = same_word_grid, same_curve_grid = same_curve_grid, 
                                           same_point_grid = same_point_grid, 
                                           mp = mp, para_estim = para_estim, para_estim_nc = para_estim_nc, 
                                           use_RI = use_RI, weights = NULL, use_simple = use_simple, np = np)) 
    }
    
    #################################
    # tri constr: using triangle only 
    # with our symmetry constraint
    #################################
    if(use_tri_constr){ 
      cat("covariance estimation tri constraint", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
      res[["time_cov_tri_constr"]] <- 
        system.time(res[["cov_hat_tri_constr"]] <- 
                      estimate_cov_tri_constr_fun(index_upperTri = index_upperTri, bf = bf_covs, method = method, 
                                                  grid_col = grid_col, grid_row = grid_row, d_grid = d_grid, 
                                                  bs = bs, m = m_covs, use_bam = use_bam, t = t, same_subject_grid = same_subject_grid, 
                                                  same_word_grid = same_word_grid, same_curve_grid = same_curve_grid, 
                                                  same_point_grid = same_point_grid, 
                                                  mp = mp, para_estim = para_estim, para_estim_nc = para_estim_nc, 
                                                  use_RI = use_RI, weights = NULL, use_simple = use_simple, np = np))  
    }
    
    ###################################################
    # tri constr weights: using triangle only 
    # with our symmetry constraint and diagonal weights
    ###################################################
    if(use_tri_constr_weights){ 
      cat("covariance estimation tri constraint with weights", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
      index_upperTri[, weights := 1]
      index_upperTri[row_t_bivariate == col_t_bivariate, weights := 0.5]
      res[["time_cov_tri_constr_weights"]] <- 
        system.time(res[["cov_hat_tri_constr_weights"]] <- 
                      estimate_cov_tri_constr_fun(index_upperTri = index_upperTri, bf = bf_covs, method = method, 
                                                  grid_col = grid_col, grid_row = grid_row, d_grid = d_grid, 
                                                  bs = bs, m = m_covs, use_bam = use_bam, t = t, same_subject_grid = same_subject_grid, 
                                                  same_word_grid = same_word_grid, same_curve_grid = same_curve_grid, 
                                                  same_point_grid = same_point_grid, 
                                                  mp = mp, para_estim = para_estim, para_estim_nc = para_estim_nc, 
                                                  use_RI = use_RI, weights = index_upperTri$weights, use_simple = use_simple, np = np)) 
    }
    
    ##########################
    # remove unnecessary stuff
    ##########################
    same_subject_grid <- NULL
    same_word_grid <- NULL
    same_curve_grid <- NULL
    same_point_grid <- NULL
    grid_row <- NULL
    grid_col <- NULL
    
    set(index, i = NULL, "row_t_bivariate", NULL)
    set(index, i = NULL, "col_t_bivariate", NULL)
    
    
    ##################################
    # check if grid is equidistant
    # if not: specify equidistant grid
    ##################################
    if(!isTRUE(all.equal(0, var(diff(res[["my_grid"]]))))){
      res[["my_grid"]] <- seq(min_grid, max_grid, length = d_grid)
      warning("the pre-specified evaluation grid was adapted to be equidistant
            for eigen decomposition an following steps")
    }
    
    
    ############################################################################
    # Functional principal component analysis
    ############################################################################
    
    if(use_RI){
      N_C <- 0  #set to zero for ease of implementation
    }
    
    #################################
    # whole: using all cross products
    # without symmetry constraint
    #################################
    if(use_whole){
      cat("fpc estimation whole", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
      if(!is.na(res[["cov_hat_whole"]][["sigmasq"]])){
        res[["time_fpc_whole"]] <- 
          system.time(res[["fpc_hat_whole"]] <- 
                        estimate_fpc_fun(cov_B = res[["cov_hat_whole"]][["grid_mat_B"]], 
                                           cov_C = res[["cov_hat_whole"]][["grid_mat_C"]], 
                                           cov_E = res[["cov_hat_whole"]][["grid_mat_E"]], 
                                           sigmasq_int_hat = res[["cov_hat_whole"]][["sigmasq_int"]], 
                                           my_grid = res[["my_grid"]], 
                                           var_level = var_level, N_B = N_B, N_C = N_C, N_E = N_E, 
                                           curve_info = curve_info,
                                           I = I, J = J, n = n, 
                                           sigmasq_hat = res[["cov_hat_whole"]][["sigmasq"]], 
                                           use_RI = use_RI))
      }else{
        res[["fpc_hat_whole"]] <- NA
      }
    }
    
    #############################
    # tri: using triangle only
    # without symmetry constraint
    #############################
    if(use_tri){ 
      cat("fpc estimation tri", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
      if(!is.na(res[["cov_hat_tri"]][["sigmasq"]])){
        res[["time_fpc_tri"]] <- 
          system.time(res[["fpc_hat_tri"]] <- 
                        estimate_fpc_fun(cov_B = res[["cov_hat_tri"]][["grid_mat_B"]], 
                                           cov_C = res[["cov_hat_tri"]][["grid_mat_C"]], 
                                           cov_E = res[["cov_hat_tri"]][["grid_mat_E"]], 
                                           sigmasq_int_hat = res[["cov_hat_tri"]][["sigmasq_int"]], 
                                           my_grid = res[["my_grid"]], 
                                           var_level = var_level, N_B = N_B, N_C = N_C, N_E = N_E, 
                                           curve_info = curve_info, I = I, J = J, n = n, 
                                           sigmasq_hat = res[["cov_hat_tri"]][["sigmasq"]], 
                                           use_RI = use_RI))
        
      }else{
        res[["fpc_hat_tri"]] <- NA
      }
    }
    
    #################################
    # tri constr: using triangle only 
    # with our symmetry constraint
    #################################
    if(use_tri_constr){
      cat("fpc estimation tri constraint", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
      if(!is.na(res[["cov_hat_tri_constr"]][["sigmasq"]])){
        res[["time_fpc_tri_constr"]] <- 
          system.time(res[["fpc_hat_tri_constr"]] <- 
                        estimate_fpc_fun(cov_B = res[["cov_hat_tri_constr"]][["grid_mat_B"]], 
                                           cov_C = res[["cov_hat_tri_constr"]][["grid_mat_C"]], 
                                           cov_E = res[["cov_hat_tri_constr"]][["grid_mat_E"]], 
                                           sigmasq_int_hat = res[["cov_hat_tri_constr"]][["sigmasq_int"]], 
                                           my_grid = res[["my_grid"]], 
                                           var_level = var_level, N_B = N_B, N_C = N_C, N_E = N_E, 
                                           curve_info = curve_info, I = I, J = J, n = n, 
                                           sigmasq_hat = res[["cov_hat_tri_constr"]][["sigmasq"]], 
                                           use_RI = use_RI))
      }else{
        res[["fpc_hat_tri_constr"]] <- NA
      }
    }
    
    ###################################################
    # tri constr: using triangle only 
    # with our symmetry constraint and diagonal weights
    ###################################################
    if(use_tri_constr_weights){
      cat("fpc estimation tri constraint weights", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
      if(!is.na(res[["cov_hat_tri_constr_weights"]][["sigmasq"]])){
        res[["time_fpc_tri_constr_weights"]] <- 
          system.time(res[["fpc_hat_tri_constr_weights"]] <- 
                        estimate_fpc_fun(cov_B = res[["cov_hat_tri_constr_weights"]][["grid_mat_B"]], 
                                           cov_C = res[["cov_hat_tri_constr_weights"]][["grid_mat_C"]], 
                                           cov_E = res[["cov_hat_tri_constr_weights"]][["grid_mat_E"]], 
                                           sigmasq_int_hat = res[["cov_hat_tri_constr_weights"]][["sigmasq_int"]], 
                                           my_grid = res[["my_grid"]], 
                                           var_level = var_level, N_B = N_B, N_C = N_C, N_E = N_E, 
                                           curve_info = curve_info, I = I, J = J, n = n, 
                                           sigmasq_hat = res[["cov_hat_tri_constr_weights"]][["sigmasq"]], 
                                           use_RI = use_RI))
      }else{
        res[["fpc_hat_tri_constr_weights"]] <- NA
      }
    }
    
    
    if(use_famm){
      ########################################################################
      # FPC-FAMM re-estimation of the mean and prediction of the basis weights
      ########################################################################
      if(use_whole){
        #################################
        # whole: using all cross products
        # without symmetry constraint
        #################################
        cat("fpc famm estimation whole", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
        if(!is.na(res[["cov_hat_whole"]][["sigmasq"]])){
          res[["time_fpc_famm_whole"]]<-
            system.time(res[["fpc_famm_hat_whole"]] <- 
                          estimate_fpc_famm_fun(curve_info = curve_info, my_grid = res[["my_grid"]],
                                                  phi_B_hat_grid = res[["fpc_hat_whole"]]$phi_B_hat_grid, 
                                                  phi_C_hat_grid = res[["fpc_hat_whole"]]$phi_C_hat_grid, 
                                                  phi_E_hat_grid = res[["fpc_hat_whole"]]$phi_E_hat_grid, 
                                                  nu_B_hat = res[["fpc_hat_whole"]]$nu_B_hat, 
                                                  nu_C_hat = res[["fpc_hat_whole"]]$nu_C_hat, 
                                                  nu_E_hat = res[["fpc_hat_whole"]]$nu_E_hat, 
                                                  t = t, N_B = res[["fpc_hat_whole"]]$N_B, 
                                                  N_C = res[["fpc_hat_whole"]]$N_C, 
                                                  N_E = res[["fpc_hat_whole"]]$N_E, 
                                                  use_bam_famm = use_bam_famm, 
                                                  num_covariates = num_covariates, 
                                                  which_interaction = which_interaction, 
                                                  n = n, interaction = interaction, 
                                                  use_RI = use_RI, method = method, bs_y_famm = bs_y_famm, 
                                                  bs_int_famm = bs_int_famm, sigmasq = res[["cov_hat_whole"]]$sigmasq, 
                                                  covariate = covariate, para_estim_famm = para_estim_famm, 
                                                  para_estim_famm_nc = para_estim_famm_nc, covariate_form = covariate_form,
                                                  save_model_famm = save_model_famm))
        }else{
          res[["fpc_famm_hat_whole"]]<-NA
        }            
      }
      
      if(use_tri){
        #############################
        # tri: using triangle only
        # without symmetry constraint
        #############################
        cat("fpc famm estimation tri", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
        if(!is.na(res[["cov_hat_tri"]][["sigmasq"]])){
          res[["time_fpc_famm_tri"]]<-
            system.time(res[["fpc_famm_hat_tri"]] <- 
                          estimate_fpc_famm_fun(curve_info = curve_info, my_grid = res[["my_grid"]],
                                                  phi_B_hat_grid = res[["fpc_hat_tri"]]$phi_B_hat_grid, 
                                                  phi_C_hat_grid = res[["fpc_hat_tri"]]$phi_C_hat_grid, 
                                                  phi_E_hat_grid = res[["fpc_hat_tri"]]$phi_E_hat_grid, 
                                                  nu_B_hat = res[["fpc_hat_tri"]]$nu_B_hat, 
                                                  nu_C_hat = res[["fpc_hat_tri"]]$nu_C_hat, 
                                                  nu_E_hat = res[["fpc_hat_tri"]]$nu_E_hat, 
                                                  t = t, N_B = res[["fpc_hat_tri"]]$N_B, 
                                                  N_C = res[["fpc_hat_tri"]]$N_C, 
                                                  N_E = res[["fpc_hat_tri"]]$N_E, 
                                                  use_bam_famm = use_bam_famm, 
                                                  num_covariates = num_covariates, 
                                                  which_interaction = which_interaction, 
                                                  n = n, interaction = interaction, 
                                                  use_RI = use_RI, method = method, bs_y_famm = bs_y_famm, 
                                                  bs_int_famm = bs_int_famm, sigmasq = res[["cov_hat_tri"]]$sigmasq, 
                                                  covariate = covariate, para_estim_famm = para_estim_famm, 
                                                  para_estim_famm_nc = para_estim_famm_nc, covariate_form = covariate_form,
                                                  save_model_famm = save_model_famm))
        }else{
          res[["fpc_famm_hat_tri"]]<-NA
        }            
      }
      
      if(use_tri_constr){
        ########################################
        # tri constr: using tri_constrangle only
        # with our symmetry constraint
        #########################################
        cat("fpc famm estimation tri constr", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
        if(!is.na(res[["cov_hat_tri_constr"]][["sigmasq"]])){
          res[["time_fpc_famm_tri_constr"]]<-
            system.time(res[["fpc_famm_hat_tri_constr"]] <- 
                          estimate_fpc_famm_fun(curve_info = curve_info, my_grid = res[["my_grid"]],
                                                  phi_B_hat_grid = res[["fpc_hat_tri_constr"]]$phi_B_hat_grid, 
                                                  phi_C_hat_grid = res[["fpc_hat_tri_constr"]]$phi_C_hat_grid, 
                                                  phi_E_hat_grid = res[["fpc_hat_tri_constr"]]$phi_E_hat_grid, 
                                                  nu_B_hat = res[["fpc_hat_tri_constr"]]$nu_B_hat, 
                                                  nu_C_hat = res[["fpc_hat_tri_constr"]]$nu_C_hat, 
                                                  nu_E_hat = res[["fpc_hat_tri_constr"]]$nu_E_hat, 
                                                  t = t, N_B = res[["fpc_hat_tri_constr"]]$N_B, 
                                                  N_C = res[["fpc_hat_tri_constr"]]$N_C, 
                                                  N_E = res[["fpc_hat_tri_constr"]]$N_E, 
                                                  use_bam_famm = use_bam_famm, 
                                                  num_covariates = num_covariates, 
                                                  which_interaction = which_interaction, 
                                                  n = n, interaction = interaction, 
                                                  use_RI = use_RI, method = method, bs_y_famm = bs_y_famm, 
                                                  bs_int_famm = bs_int_famm, sigmasq = res[["cov_hat_tri_constr"]]$sigmasq, 
                                                  covariate = covariate, para_estim_famm = para_estim_famm, 
                                                  para_estim_famm_nc = para_estim_famm_nc, covariate_form = covariate_form,
                                                  save_model_famm = save_model_famm))
        }else{
          res[["fpc_famm_hat_tri_constr"]]<-NA
        }            
      }
      
      if(use_tri_constr_weights){
        #########################################
        # tri constr weights: using triangle only
        # with our symmetry constraint
        #########################################
        cat("fpc famm estimation tri constr with weights", "; time: ", format(Sys.time(), "%a %b %d %X"), "\n", sep = "")
        if(!is.na(res[["cov_hat_tri_constr_weights"]][["sigmasq"]])){
          res[["time_fpc_famm_tri_constr_weights"]]<-
            system.time(res[["fpc_famm_hat_tri_constr_weights"]] <- 
                          estimate_fpc_famm_fun(curve_info = curve_info, my_grid = res[["my_grid"]],
                                                  phi_B_hat_grid = res[["fpc_hat_tri_constr_weights"]]$phi_B_hat_grid, 
                                                  phi_C_hat_grid = res[["fpc_hat_tri_constr_weights"]]$phi_C_hat_grid, 
                                                  phi_E_hat_grid = res[["fpc_hat_tri_constr_weights"]]$phi_E_hat_grid, 
                                                  nu_B_hat = res[["fpc_hat_tri_constr_weights"]]$nu_B_hat, 
                                                  nu_C_hat = res[["fpc_hat_tri_constr_weights"]]$nu_C_hat, 
                                                  nu_E_hat = res[["fpc_hat_tri_constr_weights"]]$nu_E_hat, 
                                                  t = t, N_B = res[["fpc_hat_tri_constr_weights"]]$N_B, 
                                                  N_C = res[["fpc_hat_tri_constr_weights"]]$N_C, 
                                                  N_E = res[["fpc_hat_tri_constr_weights"]]$N_E, 
                                                  use_bam_famm = use_bam_famm, 
                                                  num_covariates = num_covariates, 
                                                  which_interaction = which_interaction, 
                                                  n = n, interaction = interaction, 
                                                  use_RI = use_RI, method = method, bs_y_famm = bs_y_famm, 
                                                  bs_int_famm = bs_int_famm, sigmasq = res[["cov_hat_tri_constr_weights"]]$sigmasq, 
                                                  covariate = covariate, para_estim_famm = para_estim_famm, 
                                                  para_estim_famm_nc = para_estim_famm_nc, covariate_form = covariate_form,
                                                  save_model_famm = save_model_famm))
        }else{
          res[["fpc_famm_hat_tri_constr_weights"]]<-NA
        }            
      }
    }
  }else{ 
    warning("mean estimation did not succeed")
  }
  
  ###############
  # return output
  ###############
  return(res)
}
###############################################################################################################
