################################################################################
# author: Jona Cederbaum
################################################################################
# description: specification of all input parameters and call of FLMM estimation
# especially for the phonetics data.
# all input and output parameters are described here and the input is
# passed over to function call_functions_fun.
################################################################################

###############
# load packages 
###############
require(parallel)    # for parallel estimation and prediction in bam
require(mgcv)        # (at least version: 1.8-6)
require(MASS)
require(Matrix)
require(data.table) 

##############
# for FPC-FAMM
require(devtools)     # needed for installing refundDevel
require(refundDevel)  # can be installed as follows:
# devtools::install_github("refunders/refund", ref = "devel") # (at least use commit: end of january 2015)

##################
# source functions
##################
source("../call_all_functions.R")
source("../mean_estimation.R")
source("../useful_functions.R")
source("../get_cross_products.R")
source("../cov_estimation_whole.R")
source("../cov_estimation_tri.R")
source("../tri_constraint_constructor.R")
source("../cov_estimation_tri_constr.R")
source("../fpc_estimation.R")
source("../fpc_famm_estimation.R")

###########################
# create folder for results
###########################
dir.create("results")

###########################
# generate name for results
# depending on the date
###########################
dat_start <- format(Sys.time(), "_%d_%b")


################################################################################
# specify input parameters 
################################################################################


############
# which data
############
use_data <- "s_sh_acoustic_subset" # "s_sh_acoustic"

############################################################
# which scenario
## crossed fRIs: use_RI <- FALSE and use_simple <- FALSE
## one fRI: use_RI <- TRUE and use_simple <- FALSE
## independent curves: use_RI <- TRUE and use_simple <- TRUE
############################################################
use_RI <- FALSE
use_simple <- FALSE

###################
# general smoothing
###################
method <- "REML"     # estimation method for gam or bam, see ?mgcv for more details
use_bam <- TRUE      # TRUE to use function bam instead of function gam (syntax is the same, bam is faster for large data sets)
bs <- "ps"           # spline basis function type
d_grid <- 100        # pre-specified grid length for equidistant grid on which the mean, the covariance surfaces, the eigenfunctions
                     # and the functional random effects are evaluated
                     # NOTE: the length of the grid can be important for computation time
                     # (approx. quadratic influence)
min_grid <- 0        # minimum value of eqidistant grid (should approx. correspond to minimum value of time interval)
max_grid <- 1        # maximum value of eqidistant grid (should approx. correspond to maximum value of time interval)
my_grid <- NULL      # optional evaluation grid which can be specified and used instead of d_grid, min_grid, max_grid
                     # NOTE: the grid should be equidistant

#################
# mean estimation
#################
bf_mean <- 8         # dimension of basis (number of basis functions) used for f_0(t) estimation via bam/gam
bf_covariates <- 8   # dimension of basis (number of basis functions) used for covariates in mean estimation via bam/gam
m_mean <- c(2, 3)    # spline order and order of the penalty for this term of bam/gam of mean estimation
                     # e.g. c(2, 3) are cubic splines with third order difference penalty, see ?s for details

covariate <- TRUE    # TRUE to estimate covariate effects (as part of the mean function)
num_covariates <- 4  # number of covariates
covariate_form <- rep("by", num_covariates)  # form in which covariate enters the mean function 
                     ## "by": varying-coefficient (f(t)*covariate), possible for dummy coded covariates and metric covariates
                     ## "smooth": smooth effect in t and in covariate (f(t, covariate)), only for metric covariates!
                     # NOTE: metric covariates should be centered so that the global functional intercept can be interpreted as
                     # global mean function and the effect can be interpreted as a difference from the global mean
interaction <- TRUE  # TRUE to estimate interaction effects of covariates, which interactions, see which_interaction (below)
which_interaction <- matrix(c(FALSE, TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, TRUE, FALSE, 
                              FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE), byrow = TRUE, nrow = 4, ncol = 4)
                     # symmetric matrix that specifies which interactions should be considered 
                     # entry [k, l] specifies that the interaction between covariate.k and covariate.l is modeled (example below)
                     # NOTE: entries are reduntant, [l, k] should be set to the same as [k, l] (symmetric)
save_model_mean <- FALSE  # TRUE to save gam/bam object (attention: can be large!)

#######################
# covariance estimation
#######################
bf_covs <- c(5, 5, 5)     # dimension of basis (number of basis functions) used for covariance estimation via bam/gam
                          # for each functional random effect (here 3 functional random effects)
m_covs <- list(c(2, 3), c(2, 3), c(2, 3))  # marginal spline orders and marginal orders of the penalty for bam/gam of covariance estimation
                          # as only symmetric surfaces are considered: same for both directions
                          ## for crossed fRIs: list of three vectors, e.g. list(c(2, 3), c(2, 3), c(2, 3))
                          ## for one fRI: list of two vectors, e.g. list(c(2, 3), c(2, 3))
                          ## for independent curves: list of one vector, e.g. list(c(2,3))
use_whole <- TRUE         # TRUE to estimate auto-covariances using WHOLE,
                          # i.e. estimation is based on all cross products without symmetry constraint
use_tri <- TRUE           # TRUE to estimate auto-covariances using TRI,
                          # i.e. estimation is based on the triangle only without symmetry constraint
use_tri_constr <- TRUE    # TRUE to estimate auto-covariances using TRI-CONSTR,
                          # i.e. estimation is based on the triangle only with our symmetry constraint
use_tri_constr_weights <- TRUE # TRUE to estimate auto-covariances using TRI-CONSTR-WEIGHTS,
                          # i.e. estimation is based on the triangle only with our symmetry constraint
                          # and weights of 0.5 on the diagonal
np <- TRUE                # TRUE to use 'normal parameterization' for a tensor product smooth, see ?te for more details
mp <- TRUE                # FALSE to use Kronecker product penalty instead of Kronecker sum penalty
                          # with only one smoothing parameter (WHOLE and TRI), for details see ?te
                          # for TRI-CONSTR and TRI-CONSTR-WEIGHTS only one smoothing parameter is estimated anyway
                          # NOTE: when mp <- FALSE, the warning "...single penalty tensor product smooths are 
                          # deprecated and likely to be removed soon" is given out which is no problem
para_estim <- TRUE        # TRUE to parallelize estimation ofauto-covariances (only possible using bam)
para_estim_nc <- 5        # number of cores for parallelization of estimation (only possible using bam, only active if para_estim <- TRUE)

################
# FPC estimation
################
var_level <- 0.95     # pre-specified level of variance used for the choice of the number of the FPCs
                      # instead, a specific number of FPCs can be specified (see below)
N_B <- NA             # number of components for B (fRI for first grouping variable) to keep, overrides var_level, if not NA
N_C <- NA             # number of components for C (fRI for second grouping variable) to keep, overrides var_level, if not NA
N_E <- NA             # number of components for E (smooth error) to keep, overrides var_level, if not NA

#####################
# FPC-FAMM estimation
#####################
use_famm <- FALSE            # TRUE to estimate FPC-FAMM, otherwise no confidence bands for the mean effects are given out 
use_bam_famm <- TRUE         # TRUE to use function bam instead of function gam in FPC-FAMM estimation (reduces computation time for large data sets)
para_estim_famm <- FALSE     # TRUE to parallelize FPC-FAMM estimation
para_estim_famm_nc <- 5      # number of cores for parallelization of estimation (only possible using bam, only active if para_estim_famm <- TRUE)
bs_int_famm <- list(bs = "ps", k = 8, m = c(2, 3))  # specification of the smooth of f_0(t)
                             ## bs: type of basis functions, k: number of basis functions, m: order of the spline and order of the penalty
bs_y_famm <- list(bs = "ps", k = 8, m = c(2, 3))    # specification of the smooth of the covariates (in the mean function)
                             ## bs: type of basis functions, k: number of basis functions, m: order of the spline and order of the penalty
save_model_famm <- FALSE     # TRUE to save FPC-FAMM model object (attention: can be large!)

###########
# load data
###########
load(paste0(use_data, ".Rdata"))

###################
# initialize output
###################
output <- list()

################################################################################
# start estimation
################################################################################
output[["time_all"]] <- system.time(output[["results"]] <- call_functions_fun(curve_info = curve_info, use_RI = use_RI, 
                              use_simple = use_simple, method = method,  
                              use_bam = use_bam, bs = bs, mp = mp, 
                              para_estim = para_estim, para_estim_nc = para_estim_nc, 
                              min_grid = min_grid, max_grid = max_grid, 
                              my_grid = my_grid, bf_mean = bf_mean, 
                              bf_covariates = bf_covariates, 
                              m_mean = m_mean, covariate = covariate, 
                              covariate_form = covariate_form, 
                              num_covariates = num_covariates,  
                              interaction = interaction, 
                              which_interaction = which_interaction, 
                              save_model_mean = save_model_mean, 
                              bf_covs = bf_covs, m_covs = m_covs, 
                              use_whole = use_whole, use_tri = use_tri, 
                              use_tri_constr = use_tri_constr, 
                              use_tri_constr_weights = use_tri_constr_weights, 
                              np = np, var_level = var_level, 
                              N_B = N_B, N_C = N_C, N_E = N_E, 
                              use_famm = use_famm, use_bam_famm = use_bam_famm, 
                              bs_int_famm = bs_int_famm, bs_y_famm = bs_y_famm, 
                              para_estim_famm = para_estim_famm, 
                              para_estim_famm_nc = para_estim_famm_nc, 
                              save_model_famm = save_model_famm))

#################
# get sessionInfo
#################
sessionInfo <- sessionInfo()

############
# save input
############
save(bf_mean, bf_covariates, bf_covs, method, m_mean, m_covs, 
     bs, d_grid, var_level, N_B, N_C, N_E, use_bam, 
     covariate, covariate_form, num_covariates, interaction, which_interaction, 
     use_famm, mp, para_estim, para_estim_nc, use_tri, use_whole, use_tri_constr, 
     use_tri_constr_weights, use_bam_famm, 
     use_RI, para_estim_famm, para_estim_famm_nc, 
     save_model_mean, bs_int_famm, bs_y_famm, use_simple, 
     min_grid, max_grid, my_grid, sessionInfo, save_model_famm, 
     file = paste0("results/input_", use_data, dat_start, ".Rdata"))

################################################################################
# output parameters 
################################################################################
# output is a list of two elements:

## time_all: the total system.time() for calling function call_functions_fun()

## results: a list of all estimates, including:
### mean_hat: includes the estimated components of the linear predictor 
####           for dummy covariates or metric covariates with a linear effect separately (mean_pred),
####           for metric covariates with a smooth effect (mean_pred_smooth),
###            the estimated intercept (intercept)

### for each smoothing alternative X (WHOLE, TRI, TRI-CONSTR, TRI-CONSTR-W):
### cov_hat_X: includes the estimated error variance (sigmasq), 
###              the integral of the estimated error variance over the domain (sigmasq_int)
###              the estimated auto-covariance(s) evaluated on the pre-specified grid (grid_mat_B/C/E),
###              the smoothing parameter(s) for smoothing the auto-covariance(s) (sp),
###              the time for the smoothing the auto-covariance(s) only (time_cov_estim),
###              the time for evaluating the estimated auto-covariance(s) on the pre-specified grid (time_cov_pred_grid)
### time_cov_X: the total system.time() for calling function estimate_cov_X_fun()
### fpc_hat_X: the estimated rescaled eigenfunctions evaluated on the pre-specified grid (phi_B/C/E_hat_grid),
###              the estimated rescaled eigenvalues (nu_B/C/E_hat)
###              the estimated truncation numbers (N_B/C/E)
###              the estimated total variance (total_var)
###              the estimated explained variance (var_explained)
###              the predicted random basis weights xi_B/C/E_hat
### time_fpc_X: the total system.time() for calling function estimate_fpc_X_fun()

## if use_famm == TRUE, the list results additionally contains:
### fpc_famm_hat_X: including the estimated intercept (intercept), 
###                 the residuals of the FAMM estimation (residuals),
###                 the predicted basis weigts (xi_B/C/E_hat_famm),
###                 the reconstructed functional processes evaluated on the pre-specified grid (famm_predict_B/C/E),
###                 the re-estimated mean function s(t) (famm_cb_mean),
###                 possible covariate effects (famm_cb_covariate.1 etc.),
###                 possible interaction effects (famm_cb_inter_1_2 etc.)
### time_fpc_famm_X: the total system.time() for calling estimate_fpc_famm_fun()

### in addition, the list results contains:
### n_orig: curve levels as they entered the estimation
### subject_orig: subject levels as they entered the estimation
### my_grid: pre-specified grid

#############
# save output
#############
saveRDS(output, file = paste0("results/results_", use_data, dat_start, ".rds"))


################################################################################
