##############################################################################################################
# author: Jona Cederbaum
##############################################################################################################
# description: analyze application results for phonetics data using our smoothing approach or WHOLE, TRI
##############################################################################################################

###############
# load packages
###############
require(data.table)
require(scales)

##################
# source functions
#################
source("../useful_functions.R")

########################
# define plot properties
########################
cex_use <- 2            # size in plots
cex.axis_use <- 2       # size of axes
cex.lab_use <- 3        # size of labels
cex.main_use <- 3       # size of main
main_use <- ""          # plot title
line_use <- 4           # line in which the labels of the axes should be
lwd_use <- 3            # line width
cex_legend_use <- 1.8   # legend text size
cex_legend_use<-1.8     # legend text size

###############
# which results
###############
dat_start <- "_s_sh_acoustic_25_May"  # date on which the data analysis was started.

###################
# which covariances
# estimations
###################
which_covs <- c("whole", "tri", "tri_constr", "tri_constr_weights")

##############
# load results
##############
res <- readRDS(paste("results/results", dat_start, ".rds", sep = ""))

##############################
# extract estimated components
##############################
results <- res$results
res_cov <- list()
res_fpc <- list()
for(i in seq_along(which_covs)){
  res_cov[[i]] <- results[[paste0("cov_hat_", which_covs[i])]]
  res_fpc[[i]] <- results[[paste0("fpc_hat_", which_covs[i])]]
}

##################################
# extract data and evaluation grid
##################################
curve_info <- res$curve_info
my_grid <- results[["my_grid"]]

################################################################################
# make plots for all smoothing approaches separately
################################################################################

for(i in seq_along(which_covs)){
  ##############################
  # extract estimated components
  ##############################
  res_fpc_use <- res_fpc[[i]]
  res_cov_use <- res_cov[[i]]
  N_B_hat <- res_fpc_use$N_B
  N_C_hat <- res_fpc_use$N_C
  N_E_hat <- res_fpc_use$N_E
  nu_B_hat <- res_fpc_use$nu_B_hat
  nu_C_hat <- res_fpc_use$nu_C_hat
  nu_E_hat <- res_fpc_use$nu_E_hat
  sigmasq_hat <- res_cov_use$sigmasq
  phi_B_hat_grid <- res_fpc_use$phi_B_hat_grid
  phi_C_hat_grid <- res_fpc_use$phi_C_hat_grid
  phi_E_hat_grid <- res_fpc_use$phi_E_hat_grid
  cov_B_hat <- get_covs_fun(N = N_B_hat, phi = phi_B_hat_grid, nu = nu_E_hat)
  cov_C_hat <- get_covs_fun(N = N_C_hat, phi = phi_C_hat_grid, nu = nu_E_hat)
  cov_E_hat <- get_covs_fun(N = N_E_hat, phi = phi_E_hat_grid, nu = nu_E_hat)
  
  ########################
  # plot eigenfunctions
  ########################
  pdf(paste0("results/FPCs_estimated_", which_covs[i], dat_start, ".pdf"))
  par(mar = c(5.5, 7.8, 4, 2))
  if(N_B_hat > 0){
    col_FPCs_B <- c(rep("black", 2), rep(alpha("black", 0.5), 9))
    lwd_use_B <- c(rep(lwd_use, 2), rep((lwd_use - 1), 9))
    matplot(my_grid, phi_B_hat_grid, t="l", lwd = lwd_use_B, main="", 
            ylab="", xlab="", cex.axis = cex.axis_use, cex.lab = cex.lab_use, 
            cex.main = cex.main_use, col = col_FPCs_B, ylim = c(-2.4,2.4))
    mtext(side = 1, "normalized time (t)", line = line_use, cex = cex.lab_use)
    mtext(side = 2, expression(paste(phi["k"]^B, "(t)")), line = line_use, cex = cex.lab_use)
       
    if(N_B_hat >= 2){
      N_B_hat_use <- 2
    }else{
      N_B_hat_use <- N_B_hat
    }
    legend <- NULL
    for(k in 1:N_B_hat_use){
      legend <- cbind(legend, paste0("k=", k))
    }
    legend("bottomright", lty = seq_len(N_B_hat), legend = legend, 
           bty = "n", cex = cex_legend_use, lwd = lwd_use)
    abline(h = 0, lty = 3)
  }
  
  if(N_C_hat > 0){
    col_FPCs_C <- c(rep("black", 2), rep(alpha("black", 0.5), 9))
    lwd_use_C <- c(rep(lwd_use, 2), rep((lwd_use - 1), 9))
    matplot(my_grid, phi_C_hat_grid, t="l", lwd = lwd_use_C, main="", 
            ylab="", xlab="", cex.axis = cex.axis_use, cex.lab = cex.lab_use, 
            cex.main = cex.main_use, col = col_FPCs_C, ylim = c(-2.4,2.4))
    mtext(side = 1, "normalized time (t)", line = line_use, cex = cex.lab_use)
    mtext(side = 2, expression(paste(phi["k"]^C, "(t)")), line = line_use, cex = cex.lab_use)
    
    if(N_C_hat >= 2){
      N_C_hat_use <- 2
    }else{
      N_C_hat_use <- N_C_hat
    }
    legend <- NULL
    for(k in 1:N_C_hat_use){
      legend <- cbind(legend, paste0("k=", k))
    }
    legend("topright", lty = seq_len(N_C_hat), legend = legend, 
           bty = "n", cex = cex_legend_use, lwd = lwd_use)
    abline(h = 0, lty = 3)
  }
  
  if(N_E_hat > 0){
    col_FPCs_E <- c(rep("black", 2), rep(alpha("black", 0.5), 9))
    lwd_use_E <- c(rep(lwd_use, 2), rep((lwd_use - 1), 9))
    matplot(my_grid, phi_E_hat_grid, t="l", lwd = lwd_use_E, main="", 
            ylab="", xlab="", cex.axis = cex.axis_use, cex.lab = cex.lab_use, 
            cex.main = cex.main_use, col = col_FPCs_E, ylim = c(-2.4,2.4))
    mtext(side = 1, "normalized time (t)", line = line_use, cex = cex.lab_use)
    mtext(side = 2, expression(paste(phi["k"]^E, "(t)")), line = line_use, cex = cex.lab_use)
    
    if(N_E_hat >= 2){
      N_E_hat_use <- 2
    }else{
      N_E_hat_use <- N_E_hat
    }
    legend <- NULL
    for(k in 1:N_E_hat_use){
      legend <- cbind(legend, paste0("k=", k))
    }
    legend("top", lty = seq_len(N_E_hat), legend = legend, 
           bty = "n", cex = cex_legend_use, lwd = lwd_use)
    abline(h = 0, lty = 3)
  }
  dev.off()
  
  #########################
  # plot covariance surface
  #########################
  pdf(paste0("results/cov_estimated_persp_image_", which_covs[i], dat_start, ".pdf"))
  par(mar = c(5.5, 6.2, 4, 2))

  
  if(N_B_hat > 0){
    ########################################
    # persp plot of the auto-covariance of B
    
    # make z thinner for better display
    thinner <- seq(1, nrow(cov_B_hat), by = 4) # do not use all grid points to make plot with 'border' nicer
    z_plotting <- cov_B_hat[thinner, thinner]
    
    persp(z_plotting, ticktype = "detailed", xlab = "", ylab = "", zlab = "", theta = 30, phi = 38,
          xlim = c(0, 1), ylim = c(0, 1), col = get_color_persp_fun(z_plotting),
          cex.axis = cex.axis_use, main = "auto-covariance of B", cex.main = cex.main_use) 
    
    ########################################
    # image plot of the auto-covariance of B
    
    image(my_grid, my_grid, cov_B_hat, xlab = "", ylab = "", main = "auto-covariance of B", 
          cex.main = cex.main_use,
          col = terrain.colors(ncol(cov_B_hat)), cex.axis = cex.axis_use, cex.lab = cex.lab_use)
    contour(my_grid, my_grid, cov_B_hat, add = T)
    mtext(side = 1, "normalized time (t)", line = line_use, cex = cex.lab_use)
    mtext(side = 2, "normalized time (t)", line = line_use, cex = cex.lab_use)
  }
  
  if(N_C_hat > 0){
    ########################################
    # persp plot of the auto-covariance of C
    
    # make z thinner for better display
    thinner <- seq(1, nrow(cov_C_hat), by = 4) # do not use all grid points to make plot with 'border' nicer
    z_plotting <- cov_C_hat[thinner, thinner]
    
    persp(z_plotting, ticktype = "detailed", xlab = "", ylab = "", zlab = "", theta = 30, phi = 38,
          xlim = c(0, 1), ylim = c(0, 1), col = get_color_persp_fun(z_plotting),
          cex.axis = cex.axis_use, main = "auto-covariance of C", cex.main = cex.main_use) 
    
    ########################################
    # image plot of the auto-covariance of C
    
    image(my_grid, my_grid, cov_C_hat, xlab = "", ylab = "", main = "auto-covariance of C", 
          cex.main = cex.main_use, 
          col = terrain.colors(ncol(cov_C_hat)), cex.axis = cex.axis_use, cex.lab = cex.lab_use)
    contour(my_grid, my_grid, cov_C_hat, add = T)
    mtext(side = 1, "normalized time (t)", line = line_use, cex = cex.lab_use)
    mtext(side = 2, "normalized time (t)", line = line_use, cex = cex.lab_use)
    
  }
  
  if(N_E_hat > 0){
    ########################################
    # persp plot of the auto-covariance of E
    
    # make z thinner for better display
    thinner <- seq(1, nrow(cov_E_hat), by = 4) # do not use all grid points to make plot with 'border' nicer
    z_plotting <- cov_E_hat[thinner, thinner]
    
    persp(z_plotting, ticktype = "detailed", xlab = "", ylab = "", zlab = "", theta = 30, phi = 38,
          xlim = c(0, 1), ylim = c(0, 1), col = get_color_persp_fun(z_plotting),
          cex.axis = cex.axis_use, main = "auto-covariance of E", cex.main = cex.main_use) 
    
    ########################################
    # image plot of the auto-covariance of E
    
    image(my_grid, my_grid, cov_E_hat, xlab = "", ylab = "", main = "auto-covariance of E", 
          cex.main = cex.main_use, 
          col = terrain.colors(ncol(cov_E_hat)), cex.axis = cex.axis_use, cex.lab = cex.lab_use)
    contour(my_grid, my_grid, cov_E_hat, add = T)
    mtext(side = 1, "normalized time (t)", line = line_use, cex = cex.lab_use)
    mtext(side = 2, "normalized time (t)", line = line_use, cex = cex.lab_use)
    dev.off()
  }
  
}
################################################################################
