################################################################################
# author: Jona Cederbaum
################################################################################
# description: generate simulation data (with seed)
# calls function gendata in file gen_function_crossed.R
# gives out gen_normal_crossed.Rdata and gen_nonnormal_crossed.Rdata
################################################################################

################
# load packages
################
require(mvtnorm)
require(data.table)
require(orthopolynom)
require(expm)

##############
# source files
##############
source("gen_function.R")
source("ortho_functions.R")

###########################
# create folder for results
###########################
dir.create("sim_data")

################################################################################
# specify input parameters 
################################################################################

######################################################################
# which scenario
## crossed fRIs: use_RI_true <- FALSE and use_simple_true <- FALSE
## one fRI: use_RI_true <- TRUE and use_simple_true <- FALSE
## independent curves: use_RI_true <- TRUE and use_simple_true <- TRUE
######################################################################
use_RI_true <- TRUE
use_simple_true <- FALSE

################################################
# number of data sets per condition (iterations)
# number of levels per grouping variable
################################################
nrep <- 200                  # numberof data sets
Iseq <- 100                  # number of levels for first grouping variable
                             # NOTE: for simple setting, Iseq gives the number of curves
Jseq <- NA                   # number of levels for first grouping variable
                             # NOTE: for use_RI_true should be set to NA
nu_B_fun <- function(k){2/k} # function of eigenvalues for first grouping variable
                             # NOTE: for simple setting, nu_B_fun gives function of eigenvalues for smooth error
nu_C_fun <- NA               # function of eigenvalues for second grouping variable
                             # NOTE: for use_RI_true, should be set to NA
nu_E_fun <- NA               # function of eigenvalues for smooth error
                             # NOTE: for simple setting, should be set to NA
sigmasq <- 0.01              # error variance
normal <- TRUE               # TRUE to generate Gaussian random basis weights,
                             # FALSE to generate random basis weights from a mixture normal
mu <- function(t){t+sin(t)}  # mean function as function of t
N_B <- 2                     # number of eigenfunctions for first grouping variable
N_C <- NA                    # number of eigenfunctions for first grouping variable
                             # NOTE: for use_RI_true, should be set to NA
N_E <- 0                     # number of eigenfunctions for smooth error
                             # NOTE: for simple setting, should be set to 0
phi_B_fun <- phi_E           # function of eigenfunctions for first grouping variable (e.g. from ortho_functions.R)
                             # NOTE: for simple setting, phi_B_fun gives function of eigenvalues for smooth error 
phi_C_fun <- NA              # function of eigenfunctions for second grouping variable (e.g. from ortho_functions.R)
                             # NOTE: for use_RI_true, should be set to NA
phi_E_fun <- NA              # function of eigenfunctions for smooth error (e.g. from ortho_functions.R)
                             # NOTE: for simple setting, should be set to NA
minsize <- 40                # for irregular settings, minimal number of observation points per curve
maxsize <- 60                # for irregular settings, maximal number of observation points per curve
min_grid <- 0                # lower boundary of grid on which the observarion points are randomly drawn
max_grid <- 1                # upper boundary of grid on which the observarion points are randomly drawn
min_visit <- 1               ## for two crossed fRIs: minimal number of curves for combination of two grouping variables
                             ## for one fRI: minimal number of curves for grouping variable
                             # NOTE: for simple setting, should be set to 1
max_visit <- 1               ## for two crossed fRIs: maximal number of curves for combination of two grouping variables
                             ## for one fRI: maximal number of curves for grouping variable
                             # NOTE: for simple setting, should be set to 1
center_basis_weights <- TRUE # TRUE to center random basis weights
decor_basis_weights <- TRUE  # TRUE to decorrelate random basis weights

################################################################################
# generate simulation data and 
# save the data sets
################################################################################

set.seed(123)

gen <- gendata_fun(nrep = nrep, Iseq = Iseq, Jseq = Jseq, nu_B_fun = nu_B_fun, 
                   nu_C_fun = nu_C_fun, nu_E_fun = nu_E_fun, 
                 sigmasq = sigmasq, normal = normal, mu = mu, 
                 N_B = N_B, N_C = N_C, N_E = N_E, phi_B_fun = phi_B_fun, 
                 phi_C_fun = phi_C_fun, phi_E_fun = phi_E_fun, minsize = minsize, 
                 maxsize = maxsize, min_grid = min_grid, max_grid = max_grid, 
                 min_visit = min_visit, max_visit = max_visit, use_RI_true = use_RI_true, 
                 use_simple_true = use_simple_true, 
                 center_basis_weights = center_basis_weights, decor_basis_weights = decor_basis_weights)  

####################
# extract components
####################
Iseq <- gen[["Iseq"]]
if(!use_RI_true)
  Jseq <- gen[["Jseq"]]
sigmasq_true <- gen[["sigmasq_true"]]
nu_B_true <- gen[["nu_B_true"]]
if(!use_RI_true)
  nu_C_true <- gen[["nu_C_true"]]
nu_E_true <- gen[["nu_E_true"]]
phi_B_true <- gen[["phi_B_fun_true"]]
if(!use_RI_true)
  phi_C_true <- gen[["phi_C_fun_true"]]
phi_E_true <- gen[["phi_E_fun_true"]]
mu_true <- gen[["mu_true"]]
minsize_true <- gen[["minsize_true"]]
maxsize_true <- gen[["maxsize_true"]]
min_grid_true <- gen[["min_grid_true"]]
max_grid_true <- gen[["max_grid_true"]]
min_visit_true <- gen[["min_visit_true"]]
max_visit_true <- gen[["max_visit_true"]]
nrep <- gen[["nrep"]]
center_basis_weights <- gen[["center_basis_weights"]]
decor_basis_weights <- gen[["decor_basis_weights"]]


################################################################################
# save simulation data
################################################################################
if(gen[["normal"]]){
  if(use_RI_true){
    ##################
    # for crossed fRIs
    if(!use_simple_true){
      ############
      save(gen, nrep, Iseq, sigmasq_true, nu_B_true, nu_E_true, phi_B_true, 
           phi_E_true, mu_true, minsize_true, maxsize_true, min_visit_true, 
           max_visit_true, min_grid_true, max_grid_true, use_RI_true, 
           center_basis_weights = center_basis_weights, 
           decor_basis_weights = decor_basis_weights, 
           file = paste("sim_data/gen_normal_RI", paste(format(Sys.time(), "_%d_%b")), ".Rdata", sep = ""))
    }else{
      save(gen, nrep, Iseq, sigmasq_true, nu_B_true, nu_E_true, phi_B_true, 
           phi_E_true, mu_true, minsize_true, maxsize_true, min_visit_true, 
           max_visit_true, min_grid_true, max_grid_true, use_RI_true, 
           center_basis_weights = center_basis_weights, 
           decor_basis_weights = decor_basis_weights, 
           file = paste("sim_data/gen_normal_simple", paste(format(Sys.time(), "_%d_%b")), ".Rdata", sep = ""))
    }
  }else{
    save(gen, nrep, Iseq, Jseq, sigmasq_true, nu_B_true, nu_C_true, nu_E_true, 
         phi_B_true, phi_C_true, phi_E_true, mu_true, minsize_true, maxsize_true, 
         min_visit_true, max_visit_true, min_grid_true, max_grid_true, 
         use_RI_true, center_basis_weights = center_basis_weights, 
         decor_basis_weights = decor_basis_weights, 
         file = paste("sim_data/gen_normal_crossed", paste(format(Sys.time(), "_%d_%b")), ".Rdata", sep = ""))
  }  
}else{
  if(use_RI_true){
    if(!use_simple_true){
      save(gen, nrep, Iseq, sigmasq_true, nu_B_true, nu_E_true, phi_B_true, 
           phi_E_true, mu_true, minsize_true, maxsize_true, min_visit_true, 
           max_visit_true, min_grid_true, max_grid_true, 
           use_RI_true, center_basis_weights = center_basis_weights, 
           decor_basis_weights = decor_basis_weights, 
           file = paste("sim_data/gen_normal_RI", paste(format(Sys.time(), "_%d_%b")), ".Rdata", sep = ""))
    }else{
      save(gen, nrep, Iseq, sigmasq_true, nu_B_true, nu_E_true, phi_B_true, 
           phi_E_true, mu_true, minsize_true, maxsize_true, min_visit_true, 
           max_visit_true, min_grid_true, max_grid_true, 
           use_RI_true, center_basis_weights = center_basis_weights, 
           decor_basis_weights = decor_basis_weights, 
           file = paste("sim_data/gen_normal_simple", paste(format(Sys.time(), "_%d_%b")), ".Rdata", sep = ""))
    }
  }else{
    save(gen, nrep, Iseq, Jseq, sigmasq_true, nu_B_true, nu_C_true, nu_E_true, 
         phi_B_true, phi_C_true, phi_E_true, mu_true, minsize_true, maxsize_true, 
         min_visit_true, max_visit_true, min_grid_true, max_grid_true, 
         use_RI_true, center_basis_weights = center_basis_weights, 
         decor_basis_weights = decor_basis_weights, 
         file = paste("sim_data/gen_normal_crossed", paste(format(Sys.time(), "_%d_%b")), ".Rdata", sep = ""))
  }
}

###################################################################################################################
