################################################################################
# author: Jona Cederbaum
################################################################################
# description: main frame for simulation using FACE or FACE-STEP-1 
# after data have been generated
################################################################################
sim_face_fun <- function(bf, m, d_grid, var_level, dat_gen, family, min_grid, max_grid, calculate.scores = TRUE, 
                         use_step_1_only, nrep_use){
  
  for(f in family){
    ###################
    # load simulation
    # data for family f
    ###################
    load(paste0("sim_data/gen_" ,f, "_simple", dat_gen, ".Rdata"))  
    
    #########################
    # reduce iteration number
    # if desired
    #########################
    if(!is.null(nrep_use)){
      if(nrep_use <= nrep){
        nrep <- nrep_use
        warning(paste0("nrep is reduced to ", nrep))
      }else{
        warning(paste0("nrep is chosen too be higher than the existing ", nrep, " data sets "))
      }
    }
    
    ###########################
    # generate name for results
    # depending on the date
    # and type of scenario
    ###########################
    if(!use_step_1_only){
      dat_start <- paste0("_simple_face", dat_gen, format(Sys.time(),"_%d_%b"))  
    }else{
      dat_start <- paste0("_simple_face_step_1", dat_gen, format(Sys.time(),"_%d_%b"))
    }
    
    ###############################
    # iterate over number of curves 
    ###############################
    for(I in Iseq){
      Jseq <- NA  
      for(J in Jseq){
        ########################################################################
        # simulation run for
        # one iteration
        ########################################################################
        sim_r <- function(r){
          if(!file.exists(paste0("results/", f, "/res_I", I, "_J", J, "_r", r, dat_start, ".rds"))){
            cat("iteration: ", r, "; I: ", I, "; J: ", J, "; ", f , "; time: ", format(Sys.time(), "%d %b %X"), "\n", sep="")
            
            ###################
            # initialize output
            ###################
            output <- list()
            
            #######################
            # extract known 
            # components from truth
            #######################
            curve_info_true <- gen[[paste0("curve_info_true_I", I, "_J", J, "_r", r)]]
            curve_info <- data.frame(argvals = curve_info_true$t, subj = curve_info_true$n_long, y = curve_info_true$y)
            
            ######################
            # remove unknown truth
            ######################
            curve_info_true <- NULL
            
            #######################################
            # adapt input parameters to face.sparse
            #######################################
            
            # get my_grid (corresponding to argvals.new) from min_grid, max_grid and d_grid
            my_grid <- seq(min_grid, max_grid, length = d_grid)
            
            # determine number of knots from number of 
            # B-splines and use degree instead of order
            degree_covs <- m[[1]][[1]] + 1   # get degree instead of order
            k <- bf - degree_covs  # number of knots
            
            #######################
            # actual simulation run
            #######################
            
            if(!use_step_1_only){
              output[["time_all"]] <- 
                system.time(output[["results"]] <- 
                              face.sparse(data = curve_info, newdata = curve_info, 
                                          center = TRUE, argvals.new = my_grid, 
                                          knots = k, knots.option = "equally-spaced", 
                                          p = degree_covs, m=m[[1]][2], 
                                          calculate.scores = calculate.scores, pve = var_level))  
            }else{
              output[["time_all"]] <- 
                system.time(output[["results"]] <- 
                              face:::face.sparse.inner(data = curve_info, newdata = curve_info, 
                                                       center = TRUE, argvals.new = my_grid, 
                                                       knots = k, knots.option = "equally-spaced", 
                                                       p = degree_covs, m=m[[1]][2], 
                                                       calculate.scores = calculate.scores, pve = var_level))
            }
            
            #################
            # get sessionInfo
            #################
            sessionInfo <- sessionInfo()
            
            #############
            # save output
            #############
            cat("save output iteration: ", r,  "; I: ", I, "; J: ", J, "; ", f , "; time: ", format(Sys.time(), "%d %b %X"), "\n", sep = "")
            saveRDS(output, file = paste0("results/", f, "/res_I", I, "_J", J, "_r", r, dat_start, ".rds"))
            
            ###########
            #save input
            ###########
            cat("save input iteration: ", r,  "; I: ", I, "; J: ", J, "; ", f , "; time: ", format(Sys.time(), "%d %b %X"), "\n", sep = "")
            save(Iseq, Jseq, nrep, bf, m, d_grid, var_level, dat_gen, family, 
                 min_grid, max_grid, my_grid, sessionInfo, calculate.scores, use_step_1_only, 
                 file = paste0("results/", f, "/input_I", I, "_J", J, "_r", r, dat_start, ".Rdata"))
          }else{
            warning(paste0("attention: file ", f, " I_", I, "_J", J, "_r", r, dat_start, " exists already"))
          }
        }
        
        ########################################################################
        # repeat for all iterations in parallel if on unix system
        # redo each iteration up to five times in case of abort and save
        # the information on the retries.
        ########################################################################
        reps_use <- 1:nrep
        do <- 1
        while(do < 5){
          if(.Platform$OS.type == "unix"){         
            foreach(r = 1:length(reps_use)) %dopar%
              sim_r(reps_use[r])
          }else{
            foreach(r = 1:length(reps_use)) %do%
              sim_r(reps_use[r])
          }
          
          out <- numeric(nrep)
          for(r in 1:nrep){
            name_redo <- paste0("results/", f, "/res_I", I, "_J", J, "_r", r, dat_start, ".rds")
            out[r] <- file.exists(name_redo)  
          }          
          if(all(out == TRUE))
            break
          
          reps_use <- which(out == FALSE)
          warning(paste0("redo iterations ", reps_use ))
          save(reps_use, file = paste0("results/", f, "/not_worked_try", do, "_I", I, "_J", J, dat_start, ".Rdata"))
          do <- do + 1
        }    
        warnings()
      }       
    }
    gen<-NULL
    gc()
  }
}
################################################################################



